
class Node():

    def __init__(self, name="Root", parent=None):
        self.parent = parent
        if type(self.parent) == Node:
            self.parent.childs.append(self)
        self.childs = list()
        self.name = name.upper()
        self.n = self.name[0]
        self.x, self.y = 250, 0

    @property
    def dist(self):
        nb = 0
        lieu = self
        while lieu.parent != None:
            nb += 1
            lieu = lieu.parent
        return nb+1

    @property
    def ancetre(self):
        if self.parent == None:
            return self.name
        else:
            return self.parent.ancetre+'/'+self.name

    def __repr__(self):
        return self.name

    def tree(self, nb=0):
        t = self.name
        for child in self.childs:
            t += "\n"+" | "*(nb+1)+child.tree(nb+1)
        if nb == 0:
            print(t)
        else:
            return t

    def deepest(self):
        if self.childs == []:
            return 1
        else:
            return max([child.deepest() for child in self.childs])+1

    def __getitem__(self, nb): return self.childs[nb]

    def __setitem__(self, nb, val): self.childs[nb] = val


def lenght():
    todo, taille = [(r, 1)], 0
    while todo:
        nextdo = []
        for elem, niv in todo:
            for subelem in elem.childs:
                nextdo.append((subelem, niv+1))
                if niv+1 > taille:
                    taille = niv+1
        todo = nextdo[:]
    return taille

def allchilds(r):
    todo = [r]
    alls = [r]
    while todo:
        nextdo = []
        for elem in todo:
            for subelem in elem.childs:
                nextdo.append(subelem)
        todo = nextdo[:]
        alls += nextdo[:]
    return alls

def fil(n):
    nv = racine = Node()
    for i in range(n):
        nv = Node("X", nv)
    return racine

def binarytree(n):
    racine = Node()
    leafs = [racine]
    for i in range(n):
        nextleafs = []
        for node in leafs:
            nextleafs.append(Node("g", node))
            nextleafs.append(Node("d", node))
        leafs = nextleafs[:]
    return racine

def generation(k, racine):
    leafs = [racine]
    for _ in range(k):
        nxt = []
        for i in leafs:
            for child in i:
                nxt.append(child)
        leafs = nxt[:]
    return leafs

def recur_generation(racines: list, nb):
    if nb < 0:
        return [racines]
    else:
        l = []
        for i in racines:
            l += recur_generation(i, nb-1)
        return l

def tree(racine):
    geners = [racine]
    while True:  # danger
        toappend = []
        for elem in geners[-1]:
            for child in elem.childs:
                toappend.append(child)
        if len(toappend):
            geners.append(toappend)
        else:
            break
    for gen in geners:
        for i in gen:
            print(i.n, end="\t")
        print()

def commun_ancestor(a, b):
    for i in a.ancetre:
        for y in b.ancetre:
            if y == i:
                return y, i

def node_visualisation(r=binarytree(4)):
    import pygame as pg
    pg.init()
    f = pg.display.set_mode((500, 500), pg.RESIZABLE)
    fps = pg.time.Clock()
    B = 1
    font = pg.font.SysFont('consolas Bold', 20)

    while B:
        pg.display.update()
        f.fill(0)

        fps.tick(60)
        for node in allchilds(r):
            if node.parent:
                node.y = node.parent.y+100
                nbbros = len(node.parent.childs)
                index = node.parent.childs.index(node)
                node.x = node.parent.x+(nbbros-(4*index))*(5-node.dist)*50
                pg.draw.line(f, 0xffffff, (node.x, node.y),
                             (node.parent.x, node.parent.y))

            fo = font.render(node.name, True, (255, 255, 255))
            f.blit(fo, (node.x, node.y))
        for event in pg.event.get():
            if event.type == pg.QUIT:
                pg.quit()
                B = 0
            elif event.type == pg.VIDEORESIZE:
                r.x = event.w/2

if __name__=="__main__":
    r = Node('Racine')
    a = Node('Abba', r)
    b = Node('Beu', a)
    c = Node('Clo', a)
    d = Node('Damien', b)
    e = Node('Eloi', d)
    f = Node('Fr', e)
    g = Node('gontrand', a)


class Balise:
    def __init__(self, name, parent=None, arguments=dict()):
        self.name=name
        self.childs = dict()
        self.args = arguments
        self.parent=parent
        if parent:
            parent.childs[self.name]=self

    def __repr__(self):
        indent=self.nbancestors*'\t'
        return indent+f"<{self.name} {self.repr_args()}>\n"+\
                '\n'.join([str(self.childs[child])+"\n" for child in self.childs])+\
                indent+f"</{self.name}>"

    def repr_args(self):
        t=""
        for key in self.args:
            t+=f"{key}={self.args[key]} "
        return t
    @property
    def nbancestors(self):
        nb = 0
        lieu = self
        while lieu.parent != None:
            nb += 1
            lieu = lieu.parent
        return nb
if __name__=="__main__":
    a=Balise('main',None)
    Balise('test',a)



import random

import ast
class Node():
    
    def __init__(self,name="Root",parent=None):
        self.parent=parent
        if type(self.parent)==Node:
            self.parent.childs.append(self)
        self.name=name
        self.childs=list()
        self.n=self.name[0]
        self.x,self.y=250,0
        
    @property
    def dist(self):
        nb=0
        lieu=self
        while lieu.parent!=None:
            nb+=1
            lieu=lieu.parent
        return nb+1
    
    @property
    def ancetre(self):
        if self.parent==None:
            return self.name
        else:
            return self.parent.ancetre+'/'+self.name
        return nb

    def copy(self,paent=None):
        r=self.__class__(self.name,paent)

        for child in self.childs:
            child.copy(paent=r)
        
        return r
    
    def __repr__(self):
        return self.name
    
    def tree(self,nb=0):
        t=self.name
        for child in self.childs:
            t+="\n"+" | "*(nb+1)+child.tree(nb+1)
        if nb==0:print(t)
        else: return t

    
    def deepest(self):
        if self.childs==[]:
            return 1
        else:
            return max([child.deepest() for child in self.childs])+1
            
    def __getitem__(self,nb):return self.childs[nb]

    def __setitem__(self,nb,val):self.childs[nb]=val


r=Node('Racine')
a=Node('Abba',r)
b=Node('Beu',a)
c=Node('Clo',a)
d=Node('Damien',b)
e=Node('Eloi',d)
f=Node('Fr',e)
g=Node('gontrand',a)
names=open("rambouillet.csv","r").read().split("\n")

"""
from urllib import request
import json
T = json.load(request.urlopen(
    "https://raw.githubusercontent.com/dataaddict/prenoms/master/src/data/raw.json"))
years={}
for elem in T:
    name=elem['forename']
    if elem['year'] in years:
        freres=years[elem['year']]
        if not (name in freres):
            years[elem['year']].append(name)
    else:
        years[elem['year']]=[name]
"""

years= ast.literal_eval(open('years','r').read())

def random_tree(begin_year=1900):
    main=Node(random.choice(['Adam','Eve']))
    gen=[main]
    for i in range(5):
        nextgen=[]
        for elem in gen:
            for j in range(random.randint(1,5)):
                anne=begin_year+random.randrange(12,45)*i
                if anne<2016:
                    age_de_vie=random.randrange(50,80)
                    new=Node(random.choice(years[anne])+f' ({anne}-{anne+age_de_vie}, {age_de_vie}ans)',
                                        elem)
                    nextgen.append(new)
        gen=nextgen[:]
    return main
            
        
def lenght():
    todo,taille=[(r,1)],0
    while todo:
        nextdo=[]
        for elem,niv in todo:
            for subelem in elem.childs:
                nextdo.append((subelem,niv+1))
                if niv+1>taille:taille=niv+1
        todo=nextdo[:]
    return taille

def allchilds(r):
    todo=[r]
    alls=[r]
    while todo:
        nextdo=[]
        for elem in todo:
            for subelem in elem.childs:
                nextdo.append(subelem)
        todo=nextdo[:]
        alls+=nextdo[:]
    return alls

def fil(n):
    nv=racine=Node()
    for i in range(n):
        nv=Node("X",nv)
    return racine

def binarytree(n):
    racine=Node()
    leafs=[racine]
    for i in range(n):
        nextleafs=[]
        for node in leafs:
            nextleafs.append(Node("g",node))
            nextleafs.append(Node("d",node))
        leafs=nextleafs[:]
    return racine

def generation(k,racine):
    leafs=[racine]
    for _ in range(k):
        nxt=[]
        for i in leafs:
            for child in i:
                nxt.append(child)
        leafs=nxt[:]
    return leafs

def recur_generation(racines:list,nb):
    if nb<0:
        return [racines]
    else:
        l=[]
        for i in racines:
            l+=recur_generation(i,nb-1)
        return l

    
def tree(racine):
    geners=[racine]
    while True: #danger
        toappend=[]
        for elem in geners[-1]:
            for child in elem.childs:
                toappend.append(child)
        if len(toappend):
            geners.append(toappend)
        else:
            break
    for gen in geners:
        for i in gen:
            print(i.n,end="\t")
        print()



def nearest_ancestor(a,b):
    tocheck_a=[a.parent]
    tocheck_b=[b.parent]
    while True:
        next_a,next_b=[],[]
        
        for parent in tocheck_a:
            if parent in tocheck_b:
                return parent
            next_a+=[parent.parent]
            
        for parent in tocheck_b:
            if parent in tocheck_a:
                return parent
            next_b+=[parent.parent]
        print(tocheck_a,tocheck_b)
        tocheck_a,to_check_b=next_a[:],next_b[:]

"""def nearest_ancestor(a,b):
    tocheck=[[a.parent],[b.parent]]
    while True:
        next_=[[],[]]
        
        for i in range(2):
            for parent in tocheck[i]:
                if parent in tocheck[1-i]:
                    return parent
                next_[i]+=[parent.parent]
        print(tocheck)   
        tocheck[0],tocheck[1]=next_[0][:],next_[1][:]"""
        



def pygame():
    import pygame as pg
    pg.init()
    f=pg.display.set_mode((500,500),pg.RESIZABLE)
    fps=pg.time.Clock()
    B=1
    font=pg.font.SysFont('consolas Bold',20)
    #r=binarytree(4)

    while B:
        pg.display.update()
        f.fill(0)
        
        fps.tick(60)
        for node in allchilds(r):
            if node.parent:
                node.y=node.parent.y+100
                nbbros=len(node.parent.childs)
                index=node.parent.childs.index(node)
                node.x=node.parent.x+(nbbros-(4*index))*(5-node.dist)*50
                pg.draw.line(f,0xffffff,(node.x,node.y),(node.parent.x,node.parent.y))
                
                
            fo=font.render(node.name,True,(255,255,255))
            f.blit(fo,(node.x,node.y))
        for event in pg.event.get():
            if event.type==pg.QUIT:
                pg.quit()
                B=0
            elif event.type==pg.VIDEORESIZE:
                r.x=event.w/2
        

    



class Table:
    def __init__(self, size=(0, 0), default=lambda x, y: (x, y)):
        self.list = [[]]
        self.default = default
        self.addcolumn(size[0])
        self.addline(size[1]-1)

    def __getitem__(self, val):
        try:
            return self.list[val[1]][val[0]]
        except TypeError:
            if type(val) == int:
                return self.list[val]
            else:
                raise

    def __setitem__(self, pos, value):
        if pos[0] >= self.xmax:
            self.addcolumn(pos[0]-self.xmax+1)
        if pos[1] >= self.ymax:
            self.addline(pos[1]-self.ymax)
        self.list[pos[1]][pos[0]] = value

    def addcolumn(self, nb=1):
        for y in range(self.ymax+1):  # For each line
            self.list[y] += [self.default(x, y)
                             for x in range(nb)]  # Ajouter nb colone(s)

    def addline(self, nb=1):
        for y in range(self.ymax+1, self.ymax+nb+1):  # Pour each nouvelle line
            self.list.append([self.default(x, y)
                             for x in range(self.xmax+1)])  # en créer une

    def __repr__(self):
        t = ""
        for y in self.list:
            for x in y:
                t += str(x)+"\t"
        #t+=f"{self.xmax= } {self.ymax= }"
        return t[:-1]

    def __iter__(self):
        self.av = [0, 0]
        return self

    def __next__(self):
        res = self.av[:]
        if self.av == "ended":
            raise StopIteration
        self.av[0] += 1
        if self.av[0] > self.xmax:
            self.av[0] = 0
            self.av[1] += 1
            if self.av[1] > self.ymax:
                self.av = "ended"
                return self[-1, -1], res
        return self[res[0], res[1]], res

    @property
    def xmax(self):
        try:
            val = len(self.list[0])
            return val-1 if val > 0 else 0
        except IndexError:
            return 0

    @property
    def ymax(self):
        val = len(self.list)
        return val-1 if val > 0 else 0


class Plan:

    def __init__(self, inter=(-5, -5, 5, 5), default=lambda x, y: (x, y)):
        self.default = default
        self.list = []

        self.ouest, self.nord, self.est, self.sud = 0, 0, 0, 0

        self.addlineDown(abs(inter[3]))
        self.addlineUp(abs(inter[1]))
        self.addcolumnLeft(abs(inter[0]))
        self.addcolumnRight(abs(inter[2]))

    def __getitem__(self, val):
        if type(val) == tuple:
            return self.list[val[1]][abs(self.ouest)+val[0]]
        else:
            print(f"Bad index for type Plan: {val}")

    def addcolumnRight(self, nb=1):
        for index, y in enumerate(range(self.nord, self.sud)):
            self.list[index] = self.list[index] + \
                [self.default(x, y) for x in range(self.est, self.est+nb)]
        self.est += nb

    def addcolumnLeft(self, nb=1):
        for index, y in enumerate(range(self.nord, self.sud)):
            self.list[index] = [self.default(x, y) for x in range(
                self.ouest-nb, self.ouest)]+self.list[index]
        self.ouest -= nb

    def addlineUp(self, nb=1):
        self.list = [[self.default(x, y) for x in range(
            self.ouest, self.est)] for y in range(self.nord-nb, self.nord)]+self.list
        self.nord -= nb

    def addlineDown(self, nb):
        self.list = self.list+[[self.default(x, y) for x in range(
            self.ouest, self.est)] for y in range(self.sud, self.sud+nb)]
        self.sud += nb

    def __setitem__(self, pos, value):
        x, y = pos
        if not self.ouest < x < self.est:
            if x < self.ouest:
                self.addcolumnLeft(abs(x-self.ouest))
            if self.est < x:
                self.addcolumnRight(abs(x-self.est))

        if not self.nord < y < self.sud:
            if y < self.nord:
                self.addlineUp(abs(y-self.nord))
            if self.sud < y:
                self.addlineDown(abs(y-self.sud))
        self.list[abs(self.nord)+y][abs(self.ouest)+x] = value

    def __repr__(self):
        t = ""
        for y in self.list:
            t += str(y)+"\n"
        return t

    def __iter__(self):
        self.av = [self.ouest, self.nord]
        return self

    def __next__(self):
        if self.av == "end":
            raise StopIteration
        else:
            res = self[self.av[0], self.av[1]], self.av[:]
            self.av[0] += 1
            if self.av[0] > self.est:
                self.av[1] += 1
                if self.av[1] > self.sud:
                    self.av = 'end'
        return res
if __name__=="__main__":
    S = Plan()
    T = Table(default=lambda x, y: 0)
    D = Table(size=(10, 10), default=lambda x, y: x*y)



class Frac(float):
    
    def __init__(self,a:int,b:int):
        super().__init__()
        self.a=a
        self.b=b
        if self.b==0: raise ZeroDivisionError
        
    def __add__(self,other):
        if type(other)==Frac:return Frac(self.a*other.b+other.a*self.b,self.b*other.b)
        elif type(other)== int or type(other)==float:return Frac(self.a*other,self.b)
        else:raise TypeError
        
    def __mul__(self,other):
        return Frac(self.a*other.a,self.b*other.b)

    def __div__(self,other):
        if type(other)==Frac:return Frac(self.a*other.b,self.b*other.a)
        elif type(other)==int:return Frac(self.a*other.b,other)
        else:raise TypeError

    def simp(self):
        '''Euclide divison'''
        a,b,r=self.a,self.b,1
        while b:b,a,r=r,b,a%b
        self.a//=a
        self.b//=a
    
    def __repr__(self):
        up,down=str(self.a),str(self.b)
        long=max(len(up),len(down))+2
        upsace,downsace=" "*((long-len(up))//2)," "*((long-len(down))//2)
        return upsace+up+upsace+"\n"+"-"*long+"\n"+downsace+down+downsace
if __name__=="__main__":
    a=Frac(1,2)



class Complex:
    def __new__(cls,a,b):
        if b==0:
            return type(a).__new__(type(a),a)
        else:
            return super(Complex,cls).__new__(cls)
    def __init__(self, Re, Im):
        if Im:
            self.re = Re
            self.im = Im
        else:
            self.__class__=int

    def __add__(self, other):
        return Complex(self.re+other.re, self.im+other.im)

    def __repr__(self):
        return f"{self.re}{'+' if self.im>=0 else ''}{self.im}i"

    @property
    def bar(self):
        return Complex(self.re, -self.im)

    def set_im(self, val):
        if val == 0:
            del self.im
        else:
            self._im = val

    def del_im(self):
        self._im=0
        self.__init__(self.re,0)
    def intoger(int):pass
    im = property(lambda self: self._im, set_im, del_im)

if __name__=="__main__":
    a = Complex(1, 2)



import random,math
class Ant():
    def __init__(self,nbpattes=6,nbantennes=2,mandibulles=2):
        self.legs,self.antennes,self.mandi=nbpattes,nbantennes,mandibulles
        
    def __add__(self,other):
        return Ant((self.legs+other.legs)//2,
                   (self.antennes+other.antennes)//2,
                   (self.mandi+other.mandi)//2)
    
    @property
    def score(self):return self.legs+self.mandi+self.antennes
    
    def __mul__(self,other):
        win,loose= (self,other) if self.score>other.score else (other,self)
        if random.random()<1/100: win,loose=loose,win
        trophe=random.choice(list(win.__dict__.keys()))
        setattr(win,trophe,getattr(win,trophe)+1)
        setattr(loose,trophe,getattr(loose,trophe)-1)
        return trophe
    
    def __repr__(self):
        return f'''{"|"*(self.legs//2)}\n{"0"*max(self.legs//2,self.legs-self.legs//2)}{"."*self.mandi}{"-"*self.antennes}\n{"|"*(self.legs-self.legs//2)}'''
if __name__=="__main__":
    mimi=Ant()
    stephane=Ant(4,2,5)



from setuptools import setup, find_packages
import codecs
import os


VERSION = '0.6.9'
DESCRIPTION = 'Potits trucs utils on ne sait jamais'
LONG_DESCRIPTION = """
Plein de trucs sympa, voir le repo github pour plus de datails
Mais sinon dans vos projets utilisez le en tapant import utis
"""

# Setting up
setup(
    name="utis_matisse",
    version=VERSION,
    author="momoladebrouill",
    author_email="<momoladebrouill@github.io>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[],
    keywords=['python', 'maths', 'OOP'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



"""Stuufs utils pour momo, je sais même pas si je vais m'en serveir"""
import math

class Pos:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def dist(self, autrui):
        return math.sqrt((self.x-autrui.x)**2+(self.y-autrui.y)**2)

    def angle(self, autrui):
        return math.atan2(self.y-autrui.y, self.x-autrui.x)

    def tup(self):
        return (self.x, self.y)

    def __add__(self, other):
        return Pos(self.x+other.x, self.y+other.y)

    __sub__ = dist

    def __repr__(self):
        return str(vars(self))


class Vec:
    def __init__(self, x=None, y=None, angle=None, long=None, a=None, b=None):
        self.x = 0
        self.y = 0
        if x != None and y != None:
            self.x = x
            self.y = y
        elif angle != None and long != None:
            self.angle = angle
            self.long = long

        elif type(a) == Pos and type(b) == Pos:
            self.x = a.x-b.x
            self.y = a.y-b.y
        else:
            raise TypeError("No good pair of keywords")

    def __set_angle(self, val):
        long = self.long
        self.x = round(math.cos(val)*long, 9)
        self.y = round(math.sin(val)*long, 9)

    def __set_long(self, val):
        angle = self.angle
        self.x = round(math.cos(angle)*val, 9)
        self.y = round(math.sin(angle)*val, 9)

    angle = property(lambda self: 0 if self.long ==
                     0 else math.atan2(self.y, self.x)+math.pi, __set_angle)
    long = property(lambda self: math.sqrt(self.x**2+self.y**2), __set_long)

    def pointtoo(self, form: Pos, to: Pos):
        self.angle = form.angle(to)

    def __getitem__(self, ind):
        return [self.x, self.y][ind]

    def __add__(self, other):
        return Vec(x=self.x+other.x, y=self.y+other.y)

    def __sub__(self, other):
        return Vec(x=self.x-other.x, y=self.y-other.y)

    def __mul__(self, num):
        return self.x*num.x+self.y*num.y if type(num) == Vec else Vec(x=self.x*num, y=self.y*num)

    def __truediv__(self, num: int | float):
        return Vec(x=self.x/num, y=self.y/num)

    def __floordiv__(self, num: int| float):
        return Vec(x=int(self.x/num), y=int(self.y/num))

    def __repr__(self):
        return f"{self.x=} {self.y=}"
    __rmul__ = __mul__
    __radd__ = __add__








if __name__ == "__main__":
    a = Vec(x=1, y=1)
    b = Vec(long=1, angle=math.tau)
    c = Pos(0, 10)
    d = Pos(20, 20)
    
    ...
else:
    print('Services offerts par votre bien aimé capitaine µ uwu')
